/*
 *  Driver routines for BUFFALO Platform
 *
 *  Copyright (C)  BUFFALO INC.
 *
 *  This software may be used and distributed according to the terms of
 *  the GNU General Public License (GPL), incorporated herein by reference.
 *  Drivers based on or derived from this code fall under the GPL and must
 *  retain the authorship, copyright and license notice.  This file is not
 *  a complete program and may only be used when the entire operating
 *  system is licensed under the GPL.
 *
 */


#include <linux/module.h>
#include <linux/proc_fs.h>
#include <linux/init.h>
#include <asm/io.h>
#include <asm/uaccess.h>
#include "boardEnv/mvBoardEnvLib.h"
#include "boardEnv/buffalo/buffalo78100BoardEnv.h"
#include "BuffaloGpio.h"
#include "buffalo/kernevnt.h"
#include "buffalo/BuffaloUart.h"
#include "gpp/mvGppRegs.h"
#include "gpp/mvGpp.h"
#include "buffalo/buffalocore.h"

/* Globals */
#define BUFFALO_DRIVER_VER "0.02"
#define DRIVER_NAME	"Buffalo Gpio Control Driver"
#define AUTHOR	"(C) BUFFALO INC."

/* Module parameters */
MODULE_AUTHOR(AUTHOR);
MODULE_DESCRIPTION(DRIVER_NAME);
MODULE_LICENSE("GPL");
MODULE_VERSION(BUFFALO_DRIVER_VER);

/* Definitions */
//#define DEBUG
#define USE_PROC_BUFFALO
#define BUFFALO_LED_CONTROL_INFO_BLOCK_ENABLE
	
#define MAX_ALARM_NUM	100
#define MAX_INFO_NUM	100
#define MAX_POWER_NUM	9
#define MAX_FUNC_NUM	9

#define LED_ON	1
#define LED_OFF	0

#define LED_PHASE_10	1
#define LED_PHASE_1		2
#define LED_PHASE_SHORT	3
#define LED_PHASE_LONG	4

#define BUFFALO_LED_WAIT_10		(HZ * 10 / 10)	// 1 sec
#define BUFFALO_LED_WAIT_1		(HZ * 5 / 10)	// 0.5 sec
#define BUFFALO_LED_WAIT_SHORT		(HZ * 3 / 10)	// 0.3 sec
#define BUFFALO_LED_WAIT_LONG		(HZ * 10 / 10)	// 1 sec


#define SW_POWER_NS	-1
#define SW_POWER_OFF	0
#define SW_POWER_ON	1
#define SW_POWER_AUTO	2

#define SW_POLLING_INTERVAL	1000	// 1sec

#define SW_PIN_STAT_OFF		0
#define SW_PIN_STAT_ON		(BIT(BIT_PWR_SW))
#define SW_PIN_STAT_AUTO	(BIT(BIT_PWRAUTO_SW))

static struct timer_list BuffaloSwPollingTimer;

static unsigned int g_buffalo_sw_status = SW_POWER_NS;
static unsigned int g_buffalo_sw_polling_control = 0;

struct led_t{
	int mppNumber;
	int ErrorNumber;
	int Brighted;
	int Brighting;
	struct timer_list LedTimer;
} alarmLed, infoLed, powerLed, funcLed;


// ----------------------------------------------------

/* Definitions for DEBUG */
#ifdef DEBUG
 #define FUNCTRACE(x)  x
#else
 #define FUNCTRACE(x) 

#endif

#if defined LED_DEBUG
  #define LED_TRACE(x)	x
#else
  #define LED_TRACE(x)
#endif

#if defined PST_DEBUG
  #define PST_TRACE(x)	x
#else
  #define PST_TRACE(x)
#endif


static int
gpio_power_read_proc(char *page, char **start, off_t off, int count, int *eof, void *data)
{
	if (BIT_PWR_SW < 0)
		return 0;

	int len=0;
//	off_t begin=0;
//	unsigned int PowerStat=buffalo_gpio_read();
//	int i;

//	len += sprintf(page + len, "GpioStat=0x%04x\n", PowerStat);
//	len += sprintf(page + len, "BIT(%d)=0x%04x\n", BIT_POWER, BIT(BIT_POWER));
//	len += sprintf(page + len, "PowerStat=%d\n", (PowerStat & BIT(BIT_POWER))? 1:0);
//
//	if (BIT_LED_INFO >= 0)
//		len += sprintf(page + len, "LedInfo=%d\n", (PowerStat & BIT(BIT_LED_INFO))? 0:1);
//
//	if (BIT_LED_ALARM >= 0)
//		len += sprintf(page + len, "LedAlarm=%d\n", (PowerStat & BIT(BIT_LED_ALARM))? 0:1);
//
//	if (BIT_LED_FUNC >= 0)
//		len += sprintf(page + len, "LedFunc=%d\n", (PowerStat & BIT(BIT_LED_FUNC))? 0:1);
//
//	for (i=0; mvBoardGpioPinNumGet(BOARD_GPP_HDD_POWER, i) != -1; i++) {
//		if (bitHddPower[i] >= 0)
//		  len += sprintf(page + len, "HddPower%d=%d\n", i, (PowerStat & BIT(bitHddPower[i]))? 0:1);
//	}
//	
//	for (i=0; i<BUF_NV_SIZE_BIT_USB_POWER; i++) {
//		if (bitUsbPower[i] >= 0)
//		  len += sprintf(page + len, "UsbPower%d=%d\n", i, (PowerStat & BIT(bitUsbPower[i]))? 0:1);
//	}
	
//	*start = page + (offset - begin);
//	len -= (offset - begin);
//	if (len > length)
//		len = length;
//	return (len);
	if (off + count >= len)
		*eof = 1;

	if (len < off)
		return 0;

	*start = page + off;

	return ((count < len - off) ? count : len - off);
}


static int
BuffaloKatoi(const char *buffer, unsigned int count){
	#define MAX_LENGTH	16
	int i;
	int TmpNum=0;
	
	LED_TRACE(printk("%s > Entered\n", __FUNCTION__));
	
	if(count > MAX_LENGTH){
		LED_TRACE(printk("%s > Leaving\n", __FUNCTION__));
		return -1;
	}
	
	LED_TRACE(printk("%s > buffer=0x%08x\n", __FUNCTION__, buffer));
	LED_TRACE(printk("%s > *buffer=0x%08x\n", __FUNCTION__, *buffer));
	for(i=0; i<count; i++, buffer++){
		LED_TRACE(printk("%s > *buffer=0x%08x, TmpNum=%d\n", __FUNCTION__, *buffer, TmpNum));
		if(*buffer>='0' && *buffer<='9')
			TmpNum=(TmpNum * 10) + (*buffer - '0');
	}

	LED_TRACE(printk("%s > Leaving\n", __FUNCTION__));
	return TmpNum;
}


static int
BuffaloLedGetNextPhase(struct led_t *led_stat){
	LED_TRACE(printk("%s > Entered\n", __FUNCTION__));

	switch(led_stat->Brighting){
	case LED_ON:
		if((led_stat->ErrorNumber - led_stat->Brighted) > 0)
			return LED_PHASE_SHORT;
		else
			return LED_PHASE_LONG;
		break;
	case LED_OFF:
		LED_TRACE(printk("%s > led_stat->ErrorNumer=%d, led_stat->Brighted=%d\n",
				 __FUNCTION__, led_stat->ErrorNumber, led_stat->Brighted));
		if((led_stat->ErrorNumber - led_stat->Brighted) >= 10)
			return LED_PHASE_10;
		else
			return LED_PHASE_1;
		break;
	default:
		return -1;
	}
	LED_TRACE(printk("%s > Leaving\n", __FUNCTION__));
	return -1;
}


static void
BuffaloLed_StatChange(unsigned long data)
{
	struct led_t *led = (struct led_t *)data;
	int NextPhase = BuffaloLedGetNextPhase(led);
	LED_TRACE(printk("%s > Entered\n", __FUNCTION__));
	
	switch(NextPhase){
	case LED_PHASE_10:
		led->LedTimer.expires = (jiffies + BUFFALO_LED_WAIT_10);
#if defined BUFFALO_LED_CONTROL_INFO_BLOCK_ENABLE
		if (led == &infoLed && alarmLed.ErrorNumber != 0)
			bfGppOutRegBitSet(led->mppNumber);
		else
			bfGppOutRegBitClr(led->mppNumber);
#else
		bfGppOutRegBitClr(led->mppNumber);
#endif
		led->Brighted += 10;
		led->Brighting = LED_ON;
		LED_TRACE(printk("%s > change to LED_PHASE_10\n", __FUNCTION__));
		break;
	case LED_PHASE_1:
		led->LedTimer.expires = (jiffies + BUFFALO_LED_WAIT_1);
#if defined BUFFALO_LED_CONTROL_INFO_BLOCK_ENABLE
		if (led == &infoLed && alarmLed.ErrorNumber != 0)
			bfGppOutRegBitSet(led->mppNumber);
		else
			bfGppOutRegBitClr(led->mppNumber);
#else
		bfGppOutRegBitClr(led->mppNumber);
#endif
		led->Brighted += 1;
		led->Brighting = LED_ON;
		LED_TRACE(printk("%s > change to LED_PHASE_1\n", __FUNCTION__));
		break;
	case LED_PHASE_SHORT:
		led->LedTimer.expires = (jiffies + BUFFALO_LED_WAIT_SHORT);
		bfGppOutRegBitSet(led->mppNumber);
		led->Brighting = LED_OFF;
		LED_TRACE(printk("%s > change to LED_PHASE_SHORT\n", __FUNCTION__));
		break;
	case LED_PHASE_LONG:
		led->LedTimer.expires = (jiffies + BUFFALO_LED_WAIT_LONG);
		bfGppOutRegBitSet(led->mppNumber);
		led->Brighting = LED_OFF;
		led->Brighted = 0;
		LED_TRACE(printk("%s > change to LED_PHASE_LONG\n", __FUNCTION__));
		break;
	}
	add_timer(&led->LedTimer);
	LED_TRACE(printk("%s > Leaving\n", __FUNCTION__));
}


static void
BuffaloLed_BlinkStart(struct led_t *led){
	LED_TRACE(printk("%s > Entered\n", __FUNCTION__));
	led->Brighted = 0;
	led->Brighting = LED_OFF;

	init_timer(&led->LedTimer);
	led->LedTimer.expires = (jiffies + BUFFALO_LED_WAIT_SHORT);
	led->LedTimer.function = BuffaloLed_StatChange;
	led->LedTimer.data = (unsigned long)led;
	add_timer(&led->LedTimer);
	LED_TRACE(printk("%s > Leaving\n", __FUNCTION__));
}


static void
BuffaloLed_BlinkStop(struct led_t *led)
{
	LED_TRACE(printk("%s > Entered\n", __FUNCTION__));
	if (led->ErrorNumber)
		del_timer(&led->LedTimer);
	led->ErrorNumber = 0;
	led->Brighted = 0;
	led->Brighting = LED_OFF;
	LED_TRACE(printk("%s > Leaving\n", __FUNCTION__));
}


static int
BuffaloLedWriteProc(struct file *filep, const char *buffer, unsigned long count, void *data)
{
	int TmpNum=0;
	struct led_t *led = data;
	LED_TRACE(printk("%s > Entered\n", __FUNCTION__));
	TmpNum = BuffaloKatoi(buffer, count);
	LED_TRACE(printk("%s > TmpNum=%d\n", __FUNCTION__, TmpNum));
	
	// if the status buffalo_led_status.alarm.ErrorNumber is 0, force on is enable.
	if (TmpNum == 0) {
		if (strncmp(buffer, "on", 2) == 0 ) {
			LED_TRACE(printk("%s > Calling BuffaloGpio_AlarmLedEnable\n", __FUNCTION__));
			bfGppOutRegBitClr(led->mppNumber);
		}
		else if (strncmp(buffer, "off", 3) == 0) {
			LED_TRACE(printk("%s > Calling BuffaloGpio_AlarmLedDisable\n", __FUNCTION__));
			bfGppBlinkRegBitClr(led->mppNumber);
			bfGppOutRegBitSet(led->mppNumber);
		}
		else if (strncmp(buffer, "inv", 3) == 0) {
		  	bfGppOutRegBitInv(led->mppNumber);
		}
	}
	else {
		if (led->ErrorNumber ==0) {
			if ((TmpNum > 0) && (TmpNum <= MAX_ALARM_NUM)) {
				led->ErrorNumber = TmpNum;
				BuffaloLed_BlinkStart(led);
			}
			else if (TmpNum == 0) {
				BuffaloLed_BlinkStop(led);
			}
			else {
				// Nothing to do...
			}
		}
	}
	LED_TRACE(printk("%s > Leaving\n", __FUNCTION__));
	return count;
}


static int
BuffaloLedBlinkReadProc(char *page, char **start, off_t off, int count, int *eof, void *data)
{
	int len = 0;
	struct led_t *led = data;

	if (bfGppBlinkRegBitTest(led->mppNumber))
		len += sprintf(page + len, "on\n");
	else
		len += sprintf(page + len, "off\n");

	if (off + count >= len)
		*eof = 1;

	if (len < off)
		return 0;

	*start = page + off;

	return ((count < len - off) ? count : len - off);
}


static int
BuffaloLedBlinkWriteProc(struct file *filep, const char *buffer, unsigned long count, void *data)
{
	struct led_t *led = data;

	if(strncmp(buffer, "on", 2) == 0 ){
		bfGppBlinkRegBitSet(led->mppNumber);
	}else if(strncmp(buffer, "off", 3) == 0){
		bfGppBlinkRegBitClr(led->mppNumber);
	}
	LED_TRACE(printk("%s > Leaving\n", __FUNCTION__));

	return count;
}



static int
BuffaloAllLedReadProc(char *page, char **start, off_t off, int count, int *eof, void *data)
{
	int len = 0;

	if (BIT_PWR_LED >= 0) {
		if (!bfGppOutRegBitTest(BIT_PWR_LED))
			len += sprintf(page + len, "power led :on\n");
		else
			len += sprintf(page + len, "power led :off\n");
	}

	if (BIT_INFO_LED >=0) {
		if (!bfGppOutRegBitTest(BIT_INFO_LED))
			len += sprintf(page + len, "info  led :on\n");
		else
			len += sprintf(page + len, "info  led :off\n");
	}

	if (BIT_ALARM_LED >= 0) {
		if (!bfGppOutRegBitTest(BIT_ALARM_LED))
			len += sprintf(page + len, "alarm led :on\n");
		else
			len += sprintf(page + len, "alarm led :off\n");
	}

	if (BIT_FUNC_LED >= 0) {
		if(!bfGppOutRegBitTest(BIT_FUNC_LED))
			len += sprintf(page + len, "func led :on\n");
		else
			len += sprintf(page + len, "func led :off\n");
	}

	if (len == 0)
		len += sprintf(page + len, "available led not found.\n");

	if (off + count >= len)
		*eof = 1;

	if (len < off)
		return 0;

	*start = page + off;

	return ((count < len - off) ? count : len - off);
}


static int
BuffaloLedReadProc(char *page, char **start, off_t off, int count, int *eof, void *data)
{
	LED_TRACE(printk("%s > Entered\n", __FUNCTION__));

	int len = 0;
	struct led_t *led = data;

	if (led->mppNumber < 0)
		return 0;

	if (led->ErrorNumber == 0) {
		if (!bfGppOutRegBitTest(led->mppNumber))
			len += sprintf(page + len, "on\n");
		else
			len += sprintf(page + len, "off\n");
	}
	else {
#if defined BUFFALO_LED_CONTROL_INFO_BLOCK_ENABLE
		if (led == &infoLed && alarmLed.ErrorNumber != 0)
			len += sprintf(page + len, "%d(Blocked)\n", led->ErrorNumber);
		else
			len += sprintf(page + len, "%d\n", led->ErrorNumber);
#else
		len = sprintf(page, "%d\n", led->ErrorNumber);
#endif
	}

	LED_TRACE(printk("%s > Leaving\n", __FUNCTION__));

	if (len == 0)
		len += sprintf(page + len, "available led not found.\n");

	if (off + count >= len)
		*eof = 1;

	if (len < off)
		return 0;

	*start = page + off;

	return ((count < len - off) ? count : len - off);
}

static int
BuffaloAllLedWriteProc(struct file *filep, const char *buffer, unsigned long count, void *data)
{
	if(strncmp(buffer, "on", 2) == 0 ){
		BuffaloGpio_AllLedOn();
	}else if(strncmp(buffer, "off", 3) == 0){
		BuffaloGpio_AllLedOff();
	}
	return count;
}


static int
BuffaloHddReadProc(char *page, char **start, off_t off, int count, int *eof, void *data)
{
	int len = 0;

	if (bfGppOutRegBitTest(mvBoardGpioPinNumGet(BOARD_GPP_HDD_POWER, (int)data)))
		len = sprintf(page, "on\n");
	else
		len = sprintf(page, "off\n");

	if (off + count >= len)
		*eof = 1;

	if (len < off)
		return 0;

	*start = page + off;

	return ((count < len - off) ? count : len - off);
}


static int
BuffaloHddWriteProc(struct file *filep, const char *buffer, unsigned long count, void *data)
{
	if(strncmp(buffer, "on", 2) == 0 ){
		bfGppOutRegBitSet(mvBoardGpioPinNumGet(BOARD_GPP_HDD_POWER, (int)data));
	}else if(strncmp(buffer, "off", 3) == 0){
		bfGppOutRegBitClr(mvBoardGpioPinNumGet(BOARD_GPP_HDD_POWER, (int)data));
	}

	return count;
}


static int
BuffaloUsbReadProc(char *page, char **start, off_t off, int count, int *eof, void *data)
{
	int len = 0;

	if (bfGppOutRegBitTest(mvBoardGpioPinNumGet(BOARD_GPP_USB_VBUS_EN, (int)data)))
		len = sprintf(page, "on\n");
	else
		len = sprintf(page, "off\n");

	if (off + count >= len)
		*eof = 1;

	if (len < off)
		return 0;

	*start = page + off;

	return ((count < len - off) ? count : len - off);
}


static int
BuffaloUsbWriteProc(struct file *filep, const char *buffer, unsigned long count, void *data)
{
	if(strncmp(buffer, "on", 2) == 0 ){
		bfGppOutRegBitSet(mvBoardGpioPinNumGet(BOARD_GPP_USB_VBUS_EN, (int)data));
	}else if(strncmp(buffer, "off", 3) == 0){
		bfGppOutRegBitClr(mvBoardGpioPinNumGet(BOARD_GPP_USB_VBUS_EN, (int)data));
	}

	return count;
}


static int
BuffaloPowerReadProc(char *page, char **start, off_t off, int count, int *eof, void *data)
{
	int len = 0;

	if (bfGppInRegBitTest(BIT_PWR_SW))
		len = sprintf(page, "on\n");
	else
		len = sprintf(page, "off\n");

	if (off + count >= len)
		*eof = 1;

	if (len < off)
		return 0;

	*start = page + off;

	return ((count < len - off) ? count : len - off);
}


static int
BuffaloSlideSwitchReadProc(char *page, char **start, off_t off, int count, int *eof, void *data)
{
	int len = 0;

	if (use_slide_power)
		len += sprintf(page + len, "1\n");
	else
		len += sprintf(page + len, "0\n");

	if (off + count >= len)
		*eof = 1;

	if (len < off)
		return 0;

	*start = page + off;

	return ((count < len - off) ? count : len - off);
}


static int
BuffaloAutoPowerReadProc(char *page, char **start, off_t off, int count, int *eof, void *data)
{
	int len = 0;

	if (bfGppInRegBitTest(BIT_PWRAUTO_SW))
		len = sprintf(page, "on\n");
	else
		len = sprintf(page, "off\n");

	if (off + count >= len)
		*eof = 1;

	if (len < off)
		return 0;

	*start = page + off;

	return ((count < len - off) ? count : len - off);
}


//#define SW_POLL_DEBUG
#ifdef SW_POLL_DEBUG
 #define SW_POLL_TRACE(x)	(x)
#else
 #define SW_POLL_TRACE(x)	/* */
#endif

static int
BuffaloSwPollingCheck(void)
{
	unsigned int present_sw_status = SW_POWER_NS;
	static char msg[32];

	SW_POLL_TRACE(printk("BIT_PWRAUTO_SW = %d\n", bfGppInRegBitTest(BIT_PWRAUTO_SW)));

	if (use_slide_power) {
		if (!(bfGppInRegBitTest(BIT_PWR_SW)) && !(bfGppInRegBitTest(BIT_PWRAUTO_SW)))
			present_sw_status = SW_POWER_OFF;
		else if ((bfGppInRegBitTest(BIT_PWR_SW)) && !(bfGppInRegBitTest(BIT_PWRAUTO_SW)))
			present_sw_status = SW_POWER_ON;
		else if (!(bfGppInRegBitTest(BIT_PWR_SW)) && (bfGppInRegBitTest(BIT_PWRAUTO_SW)))
			present_sw_status = SW_POWER_AUTO;
		else
			present_sw_status = SW_POWER_NS;
	}
	else {
		if (bfGppInRegBitTest(BIT_PWRAUTO_SW))
			present_sw_status = SW_POWER_AUTO;
		else
			present_sw_status = SW_POWER_ON;
	}

	SW_POLL_TRACE(printk("present_sw_status = 0x%08x\n", present_sw_status));
	SW_POLL_TRACE(printk("g_buffalo_sw_status = 0x%08x\n", g_buffalo_sw_status));
	
	if(g_buffalo_sw_status != present_sw_status)
	{
		g_buffalo_sw_status = present_sw_status;
		memset(msg, 0, sizeof(msg));
		switch(g_buffalo_sw_status)
		{
			case SW_POWER_OFF:
				SW_POLL_TRACE(printk("%s> SW_POWER_OFF\n", __FUNCTION__));
				buffalo_kernevnt_queuein("PSW_off");;
				break;
			case SW_POWER_ON:
				SW_POLL_TRACE(printk("%s> SW_POWER_ON\n", __FUNCTION__));
				buffalo_kernevnt_queuein("PSW_on");;
				break;
			case SW_POWER_AUTO:
				SW_POLL_TRACE(printk("%s> SW_POWER_AUTO\n", __FUNCTION__));
				buffalo_kernevnt_queuein("PSW_auto");;
				break;
			case SW_POWER_NS:
				SW_POLL_TRACE(printk("%s> SW_POWER_NS\n", __FUNCTION__));
				buffalo_kernevnt_queuein("PSW_unknown");;
				break;
			default:
				break;
		}
		SW_POLL_TRACE(printk("%s\n", msg));
	}

	BuffaloSwPollingTimer.expires = (jiffies + SW_POLLING_INTERVAL);
	add_timer(&BuffaloSwPollingTimer);
	return 0;
}

static int
BuffaloSwPollingStop(void)
{
	del_timer(&BuffaloSwPollingTimer);
	return 0;
}

static int
BuffaloSwPollingStart(void)
{
	init_timer(&BuffaloSwPollingTimer);
	BuffaloSwPollingTimer.expires = (jiffies + SW_POLLING_INTERVAL);
	BuffaloSwPollingTimer.function = &BuffaloSwPollingCheck;
	BuffaloSwPollingTimer.data = 0;
	add_timer(&BuffaloSwPollingTimer);
	return 0;
}


static int
BuffaloSwPollingWriteProc(struct file *filep, const char *buffer, unsigned long count, void *data)
{
	FUNCTRACE(printk("%s> Entered\n", __FUNCTION__));
	if(strncmp(buffer, "on", 2) == 0 )
	{
		if(g_buffalo_sw_polling_control == 0)
		{
			g_buffalo_sw_polling_control = 1;
			g_buffalo_sw_status = SW_POWER_NS;
			BuffaloSwPollingStart();
		}
	}
	else if(strncmp(buffer, "off", 3) == 0)
	{
		if(g_buffalo_sw_polling_control == 1)
		{
			g_buffalo_sw_polling_control = 0;
			BuffaloSwPollingStop();
			g_buffalo_sw_status = SW_POWER_NS;
		}
	}
	return count;
}

static int
BuffaloSwPollingReadProc(char *page, char **start, off_t off, int count, int *eof, void *data)
{
	int len = 0;

	FUNCTRACE(printk("%s> Entered\n", __FUNCTION__));

	if(g_buffalo_sw_polling_control == 1)
	{
		len = sprintf(page, "on\n");
	}
	else
	{
		len = sprintf(page, "off\n");
	}

	if (off + count >= len)
		*eof = 1;

	if (len < off)
		return 0;

	*start = page + off;

	return ((count < len - off) ? count : len - off);
}



static int
BuffaloFuncReadProc(char *page, char **start, off_t off, int count, int *eof, void *data)
{
	int len = 0;

	if (bfGppInRegBitTest(BIT_FUNC_SW))
		len = sprintf(page, "on\n");
	else
		len = sprintf(page, "off\n");

	if (off + count >= len)
		*eof = 1;

	if (len < off)
		return 0;

	*start = page + off;

	return ((count < len - off) ? count : len - off);
}



static int
BuffaloEthLedReadProc(char *page, char **start, off_t off, int count, int *eof, void *data)
{
	return 0;
}


static int
BuffaloEthLedWriteProc(struct file *filep, const char *buffer, unsigned long count, void *data){
	if(strncmp(buffer, "on", 2) == 0){
		BuffaloGpio_EthLedOn();
	}else if(strncmp(buffer, "off", 3) == 0){
		BuffaloGpio_EthLedOff();
	}
	return count;
}



static int
BuffaloFanStatusReadProc(char *page, char **start, off_t off, int count, int *eof, void *data)
{
	if (BIT_FAN_LOCK < 0)
		return 0;

	int len = 0;

	if (bfGppInRegBitTest(BIT_FAN_LOCK))
		len = sprintf(page, "on\n");
	else
		len = sprintf(page, "off\n");

	if (off + count >= len)
		*eof = 1;

	if (len < off)
		return 0;

	*start = page + off;

	return ((count < len - off) ? count : len - off);
}



static int
BuffaloFanStatusWriteProc(struct file *filep, const char *buffer, unsigned long count, void *data)
{
	if(strncmp(buffer, "on", 2) == 0){
		//Nothing to do ...
	}else if(strncmp(buffer, "off", 3) == 0){
		//Nothing to do ...
	}
	return count;
}


#ifdef CONFIG_ARCH_FEROCEON_MV78XX0
static int
BuffaloModemControlForceTxReadProc(char *page, char **start, off_t off, int count, int *eof, void *data)
{
	int len = 0;

	if(bfGppOutRegBitTest(BIT_UPS_TXD))
		len += sprintf(page, "1\n");
	else
		len += sprintf(page, "0\n");

	if (off + count >= len)
		*eof = 1;

	if (len < off)
		return 0;

	*start = page + off;

	return ((count < len - off) ? count : len - off);
}

static int
BuffaloModemControlForceTxWriteProc(struct file *filep, const char *buffer, unsigned long count, void *data)
{
	if(strncmp(buffer, "1", strlen("1")) == 0)
	{
		bfGppOutRegBitSet(BIT_UPS_TXD);
	}
	else if(strncmp(buffer, "0", strlen("0")) == 0)
	{
		bfGppOutRegBitClr(BIT_UPS_TXD);
	}
	return count;
}

static int
BuffaloModemControlReadProc(char *page, char **start, off_t off, int count, int *eof, void *data)
{
	int len = 0;
	unsigned int regval = bfRegGet(GPP_MPP_CTL(3), 0x0000FFFF);

	if(regval == 0x00004444)
		len += sprintf(page, "mpp\n");
	else if(regval == 0x00000000)
		len += sprintf(page, "gpio\n");
	else
		len += sprintf(page, "unknown\n");

	if (off + count >= len)
		*eof = 1;

	if (len < off)
		return 0;

	*start = page + off;

	return ((count < len - off) ? count : len - off);
}

static int
BuffaloModemControlWriteProc(struct file *filep, const char *buffer, unsigned long count, void *data)
{
	if(strncmp(buffer, "gpio", strlen("gpio")) == 0)
	{
		// change mpp pin setting to gpio mode;
		bfRegSet(GPP_MPP_CTL(3), 0x0000FFFF, 0x00000000);
		BuffaloMctrlMode = BUFFALO_MODEM_CTRL_MODE_GPIO;
	}
	else if(strncmp(buffer, "mpp", strlen("mpp")) == 0)
	{
		// change mpp pin setting to mpp mode;
		bfRegSet(GPP_MPP_CTL(3), 0x0000FFFF, 0x00004444);
		BuffaloMctrlMode = BUFFALO_MODEM_CTRL_MODE_MPP;
	}
	return count;
}
#endif // of CONFIG_ARCH_FEROCEON_MV78XX0

static int
BuffaloFanControlWriteProc(struct file *filep, const char *buffer, unsigned long count, void *data)
{
	if(strncmp(buffer, "stop", strlen("stop")) == 0)
	{
		bfGppOutRegBitSet(BIT_FAN_LOW);
		bfGppOutRegBitSet(BIT_FAN_HIGH);
	}
	else if(strncmp(buffer, "slow", strlen("slow")) == 0)
	{
		bfGppOutRegBitSet(BIT_FAN_LOW);
		bfGppOutRegBitClr(BIT_FAN_HIGH);
	}
	else if(strncmp(buffer, "fast", strlen("fast")) == 0)
	{
		bfGppOutRegBitClr(BIT_FAN_LOW);
		bfGppOutRegBitSet(BIT_FAN_HIGH);
	}
	else if(strncmp(buffer, "full", strlen("full")) == 0)
	{
		bfGppOutRegBitClr(BIT_FAN_LOW);
		bfGppOutRegBitClr(BIT_FAN_HIGH);
	}

	return count;
}

static int
BuffaloFanControlReadProc(char *page, char **start, off_t off, int count, int *eof, void *data)
{
	int len = 0;

	if (bfGppInRegBitTest(BIT_FAN_LOW) && bfGppInRegBitTest(BIT_FAN_HIGH)) {
		len += sprintf(page, "stop\n");
	}
	else if (bfGppInRegBitTest(BIT_FAN_LOW) && !bfGppInRegBitTest(BIT_FAN_HIGH)) {
		len += sprintf(page, "slow\n");
	}
	else if (!bfGppInRegBitTest(BIT_FAN_LOW) && bfGppInRegBitTest(BIT_FAN_HIGH)) {
		len += sprintf(page, "fast\n");
	}
	else {
		len += sprintf(page, "full\n");
	}

	if (off + count >= len)
		*eof = 1;

	if (len < off)
		return 0;

	*start = page + off;

	return ((count < len - off) ? count : len - off);
}

static int BuffaloCpuStatusReadProc(char *page, char **start, off_t off, int count, int *eof, void *data)
{
	int len = 0;
	unsigned int CpuStatus = BuffaloGpio_ChangePowerStatus(0);

	switch(CpuStatus){
	case MagicKeyReboot:
		len = sprintf(page, "reboot\n");
		break;
	case MagicKeyRebootUbootPassed:
		len = sprintf(page, "reboot_uboot_passed\n");
		break;
	case MagicKeyNormalState:
		len = sprintf(page, "normal_state\n");
		break;
	case MagicKeyHwPoff:
		len = sprintf(page, "hwpoff\n");
		break;
	case MagicKeySwPoff:
		len = sprintf(page, "swpoff\n");
		break;
	case MagicKeySWPoffUbootPassed:
		len = sprintf(page, "swpoff_uboot_passed\n");
		break;
	case MagicKeyFWUpdating:
		len = sprintf(page, "fwup\n");
		break;
	case MagicKeyUpsShutdown:
		len = sprintf(page, "ups_shutdown\n");
		break;
	default:
		len = sprintf(page, "Unknown(CpuStatus=%d)\n", CpuStatus);
		break;
	}

	if (off + count >= len)
		*eof = 1;

	if (len < off)
		return 0;

	*start = page + off;

	return ((count < len - off) ? count : len - off);
}

static int BuffaloCpuStatusWriteProc(struct file *filep, const char *buffer, unsigned long count, void *data){
	
	if(strncmp(buffer, "reboot", 6) == 0){
		PST_TRACE(printk("%s > setting reboot\n", __FUNCTION__));
		BuffaloGpio_ChangePowerStatus(POWER_STATUS_REBOOTING);
	}else if(strncmp(buffer, "reboot_uboot_passed", 19) == 0){
		PST_TRACE(printk("%s > setting Reboot_uboot_passed\n", __FUNCTION__));
		BuffaloGpio_ChangePowerStatus(POWER_STATUS_REBOOTING_UBOOT_PASSED);
	}else if(strncmp(buffer, "normal_state", 12) == 0){
		PST_TRACE(printk("%s > setting normal_state\n", __FUNCTION__));
		BuffaloGpio_ChangePowerStatus(POWER_STATUS_NORMAL_STATE);
	}else if(strncmp(buffer, "hwpoff", 6) == 0){
		PST_TRACE(printk("%s > setting hwpoff\n", __FUNCTION__));
		BuffaloGpio_ChangePowerStatus(POWER_STATUS_HW_POWER_OFF);
	}else if(strncmp(buffer, "swpoff", 6) == 0){
		PST_TRACE(printk("%s > setting swpoff\n", __FUNCTION__));
		BuffaloGpio_ChangePowerStatus(POWER_STATUS_SW_POWER_OFF);
	}else if(strncmp(buffer, "swpoff_uboot_passed", 19) == 0){
		PST_TRACE(printk("%s > setting swpoff_uboot_passed\n", __FUNCTION__));
		BuffaloGpio_ChangePowerStatus(POWER_STATUS_SW_POFF_UBOOT_PASSED);
	}else if(strncmp(buffer, "fwup", 4) == 0){
		PST_TRACE(printk("%s > setting fwup\n", __FUNCTION__));
		BuffaloGpio_ChangePowerStatus(POWER_STATUS_FWUPDATING);
	}else if(strncmp(buffer, "ups_shutdown", 12) == 0){
		PST_TRACE(printk("%s > setting ups_shutdown\n", __FUNCTION__));
		BuffaloGpio_ChangePowerStatus(POWER_STATUS_UPS_SHUTDOWN);
	}else{
		PST_TRACE(printk("%s > no meaning...(%s)\n", __FUNCTION__, buffer));
	}

	return count;
}

static int
bfRegReadProc(char *page, char **start, off_t off, int count, int *eof, void *data)
{
	int len = 0;

	len += sprintf(page, "0x%08x\n", MV_REG_READ((MV_U32)data));

	if (off + count >= len)
		*eof = 1;

	if (len < off)
		return 0;

	*start = page + off;

	return ((count < len - off) ? count : len - off);
}

static int
bfRegWriteProc(struct file *filep, const char *buffer, unsigned long count, void *data)
{
	MV_REG_WRITE((MV_U32)data, simple_strtoul(buffer, NULL, 16));
	return count;
}


static int
BuffaloPMHddPowerReadProc(char *page, char **start, off_t offset, int length, int *eof, void *data)
{
	uint32_t pm_hdd_power;
	if (BuffaloReadPmGpio(&pm_hdd_power) != 0)
		return 0;

	pm_hdd_power = (pm_hdd_power & 0xff00) >> 8;

	if (pm_hdd_power & BIT((int)data))
		return sprintf(page, "on\n");
	else
		return sprintf(page, "off\n");
}


static int
BuffaloPMHddPowerWriteProc(struct file *filep, const char *buffer, unsigned long count, void *data)
{
	uint32_t pm_hdd_power;
	
	if (BuffaloReadPmGpio(&pm_hdd_power) != 0)
		return 0;

	if (strncmp(buffer, "on", 2) == 0 ){
		pm_hdd_power |= BIT((int)data + 8);
	}
	else if(strncmp(buffer, "off", 3) == 0){
		pm_hdd_power &= ~BIT((int)data + 8);
	}

	if (BuffaloWritePmGpio(&pm_hdd_power) != 0)
		return 0;

	return count;
}


static int
BuffaloPMHddDiagLedReadProc(char *page, char **start, off_t offset, int length, int *eof, void *data)
{
	uint32_t pm_diag_led;
	
	if (BuffaloReadPmGpio(&pm_diag_led) != 0)
		return 0;

	pm_diag_led = (pm_diag_led & 0xff00) >> 12;

	if (pm_diag_led & BIT((int)data))
		return sprintf(page, "off\n");
	else
		return sprintf(page, "on\n");
}


static int
BuffaloPMHddDiagLedWriteProc(struct file *filep, const char *buffer, unsigned long count, void *data)
{
	uint32_t pm_diag_led;

	if (BuffaloReadPmGpio(&pm_diag_led) != 0)
		return 0;

	if (strncmp(buffer, "on", 2) == 0 ){
		pm_diag_led &= ~BIT((int)data + 12);
	}
	else if(strncmp(buffer, "off", 3) == 0){
		pm_diag_led |= BIT((int)data + 12);
	}

	if (BuffaloWritePmGpio(&pm_diag_led) != 0)
		return 0;

	return count;
}


struct bf_proc_entry_tab proc_ent_tab[] = {
  {"buffalo/gpio/fan",              1,NULL,                    NULL,                     NULL,     -1},
  {"buffalo/gpio/power_control",    1,NULL,                    NULL,                     NULL,     -1},
  {"buffalo/cpu_status",            0,BuffaloCpuStatusReadProc,BuffaloCpuStatusWriteProc,NULL,     -1},
  {"buffalo/power_sw",              0,gpio_power_read_proc,    NULL,                     NULL,     -1},
  {"buffalo/gpio/led",              1,NULL,                    NULL,                     NULL,     -1},
  {"buffalo/gpio/led/all",          0,BuffaloAllLedReadProc,   BuffaloAllLedWriteProc,   NULL,     -1},
  {"buffalo/gpio/led/alarm",        0,BuffaloLedReadProc,      BuffaloLedWriteProc,      &alarmLed,BOARD_GPP_ALARM_LED},
  {"buffalo/gpio/led/alarm_blink",  0,BuffaloLedReadProc,      BuffaloLedWriteProc,      &alarmLed,BOARD_GPP_ALARM_LED},
  {"buffalo/gpio/led/info",         0,BuffaloLedReadProc,      BuffaloLedWriteProc,      &infoLed, BOARD_GPP_INFO_LED},
  {"buffalo/gpio/led/info_blink",   0,BuffaloLedReadProc,      BuffaloLedWriteProc,      &infoLed, BOARD_GPP_INFO_LED},
  {"buffalo/gpio/led/power",        0,BuffaloLedReadProc,      BuffaloLedWriteProc,      &powerLed,BOARD_GPP_PWR_LED},
  {"buffalo/gpio/led/power_blink",  0,BuffaloLedReadProc,      BuffaloLedWriteProc,      &powerLed,BOARD_GPP_PWR_LED},
  {"buffalo/gpio/led/func",         0,BuffaloLedReadProc,      BuffaloLedWriteProc,      &funcLed, BOARD_GPP_FUNC_LED},
  {"buffalo/gpio/led/func_blink",   0,BuffaloLedReadProc,      BuffaloLedWriteProc,      &funcLed, BOARD_GPP_FUNC_LED},
  {"buffalo/gpio/switch",           1,NULL,                    NULL,                     NULL,     -1},
  {"buffalo/gpio/switch/auto_power",0,BuffaloAutoPowerReadProc,NULL,                     NULL,     BOARD_GPP_PWRAUTO_SW},
  {"buffalo/gpio/switch/sw_control",0,BuffaloSwPollingReadProc,BuffaloSwPollingWriteProc,NULL,     BOARD_GPP_PWRAUTO_SW},
  {"buffalo/gpio/switch/func",      0,BuffaloFuncReadProc     ,NULL,                     NULL,     BOARD_GPP_FUNC_SW},
  {"buffalo/gpio/fan/lock",         0,BuffaloFanStatusReadProc,BuffaloFanStatusWriteProc,NULL,     BOARD_GPP_FAN_LOCK},
  {"buffalo/regs",                  1,NULL,                    NULL,                     NULL,     -1},
  {"buffalo/regs/gpp",              1,NULL,                    NULL,                     NULL,     -1},
  {"buffalo/regs/gpp/data_out",     0,bfRegReadProc,           bfRegWriteProc,          (void*)GPP_DATA_OUT_REG(0),-1},
  {"buffalo/regs/gpp/high_data_out",0,bfRegReadProc,           bfRegWriteProc,          (void*)GPP_DATA_OUT_REG(1),-1},
  {"buffalo/regs/gpp/data_out_enable",0,bfRegReadProc,         bfRegWriteProc,          (void*)GPP_DATA_OUT_EN_REG(0),-1},
  {"buffalo/regs/gpp/high_data_out_enable",0,bfRegReadProc,    bfRegWriteProc,          (void*)GPP_DATA_OUT_EN_REG(1),-1},
  {"buffalo/regs/gpp/blink_enable", 0,bfRegReadProc,           bfRegWriteProc,          (void*)GPP_BLINK_EN_REG(0),-1},
  {"buffalo/regs/gpp/high_blink_enable",0,bfRegReadProc,       bfRegWriteProc,          (void*)GPP_BLINK_EN_REG(1),-1},
  {"buffalo/regs/gpp/data_in_polarity",0,bfRegReadProc,        bfRegWriteProc,          (void*)GPP_DATA_IN_POL_REG(0),-1},
  {"buffalo/regs/gpp/high_data_in_polarity",0,bfRegReadProc,   bfRegWriteProc,          (void*)GPP_DATA_IN_POL_REG(1),-1},
  {"buffalo/regs/gpp/data_in",      0,bfRegReadProc,           bfRegWriteProc,          (void*)GPP_DATA_IN_REG(0),-1},
  {"buffalo/regs/gpp/high_data_in", 0,bfRegReadProc,           bfRegWriteProc,          (void*)GPP_DATA_IN_REG(1),-1},
  {"buffalo/regs/gpp/int_cause",    0,bfRegReadProc,           bfRegWriteProc,          (void*)GPP_INT_CAUSE_REG(0),-1},
  {"buffalo/regs/gpp/cpu0_edge_int_mask", 0,bfRegReadProc,     bfRegWriteProc,          (void*)GPP_INT_MASK_REG(0),-1},
  {"buffalo/regs/gpp/cpu0_level_int_mask",0,bfRegReadProc,     bfRegWriteProc,          (void*)GPP_INT_LVL_REG(0),-1},
  {"buffalo/regs/int",              1,NULL,                    NULL,                     NULL,     -1},
  {"buffalo/regs/int/main_int_low", 0,bfRegReadProc,           bfRegWriteProc,          (void*)CPU_INT_LOW_REG(0), -1},
  {"buffalo/regs/int/main_int_high",0,bfRegReadProc,           bfRegWriteProc,          (void*)CPU_INT_HIGH_REG(0), -1},
  {"buffalo/regs/int/irq_int_mask_low",0,bfRegReadProc,        bfRegWriteProc,          (void*)CPU_INT_MASK_LOW_REG(0), -1},
  {"buffalo/regs/int/irq_int_mask_high",0,bfRegReadProc,       bfRegWriteProc,          (void*)CPU_INT_MASK_HIGH_REG(0), -1},
  {NULL,                            0,NULL,                    NULL,                     NULL,     -1},
};
  
static void BuffaloProcEntryInit(void)
{
	int i;
	struct proc_dir_entry *ent;

	for (i = 0; proc_ent_tab[i].name != NULL; i++) {
		if (proc_ent_tab[i].isDirectory) {
			proc_mkdir(proc_ent_tab[i].name, 0);
		}
		else {
			if ((proc_ent_tab[i].class != -1) &&
			    (mvBoardGpioPinNumGet(proc_ent_tab[i].class, 0) == MV_ERROR)) {
				continue;
			}

			ent = create_proc_entry(proc_ent_tab[i].name, 0, 0);
			if (!ent) continue;

			if (proc_ent_tab[i].read_proc)
				ent->read_proc = proc_ent_tab[i].read_proc;

			if (proc_ent_tab[i].write_proc)
				ent->write_proc = proc_ent_tab[i].write_proc;

			ent->data = proc_ent_tab[i].data;
		}
	}
}

static void BuffaloHddPowerProcInit(void)
{
	int i;
	char buf[64];
	struct proc_dir_entry *ent;

	for (i=0; mvBoardGpioPinNumGet(BOARD_GPP_HDD_POWER, i) != -1; i++) {
		sprintf(buf, "buffalo/gpio/power_control/hdd%d", i);
		ent = create_proc_entry(buf, 0, 0);
		if (ent != NULL) {
			ent->read_proc = BuffaloHddReadProc;
			ent->write_proc= BuffaloHddWriteProc;
			ent->owner = THIS_MODULE;
			ent->data = (void *)i;
		}
	}
}

static void BuffaloUsbPowerProcInit(void)
{
	int i;
	char buf[64];
	struct proc_dir_entry *ent;

	for (i=0; mvBoardGpioPinNumGet(BOARD_GPP_USB_VBUS_EN, i) != -1; i++) {
		sprintf(buf, "buffalo/gpio/power_control/usb%d", i);
		ent = create_proc_entry(buf, 0, NULL);
		if (ent != NULL) {
			ent->read_proc = BuffaloUsbReadProc;
			ent->write_proc = BuffaloUsbWriteProc;
			ent->owner = THIS_MODULE;
			ent->data = (void *)i;
		}
	}
}

//#define FUNCTRACE(x)	x	// @@debug
//----------------------------------------------------------------------
int __init buffaloGpioDriver_init (void)
{
	struct proc_dir_entry *ent;

	FUNCTRACE(printk("%s > Entered.\n",__FUNCTION__));

	//	BuffaloGpio_FanControlWrite(BUFFALO_GPIO_FAN_SLOW);
	BuffaloFanControlWriteProc(NULL, "slow", 0, NULL);

	ent = proc_mkdir("buffalo/gpio", 0);
	if (!ent) {
		proc_mkdir("buffalo", 0);
		ent = proc_mkdir("buffalo/gpio", 0);
	}

	BuffaloProcEntryInit();
	BuffaloHddPowerProcInit();
	BuffaloUsbPowerProcInit();

	if (use_slide_power) {
		ent = create_proc_entry("buffalo/gpio/switch/power", 0, 0);
		ent->read_proc = BuffaloPowerReadProc;

		ent = create_proc_info_entry("buffalo/gpio/switch/slide_switch", 0, 0, BuffaloSlideSwitchReadProc);
	}

	if (BIT_FAN_LOW >= 0 && BIT_FAN_HIGH >= 0) {
		ent = create_proc_entry("buffalo/gpio/fan/control", 0, 0);
		ent->read_proc = BuffaloFanControlReadProc;
		ent->write_proc= BuffaloFanControlWriteProc;
	}

#ifdef CONFIG_ARCH_FEROCEON_MV78XX0
	// for switch mctrl mpp mode(RTS/CTS/TxD/RxD)
	// switchable to use as function pin / gpio pin;
	ent = proc_mkdir("buffalo/gpio/mctrl", 0);

	ent = create_proc_entry("buffalo/gpio/mctrl/mctrl_mode", 0, 0);
	ent->read_proc = BuffaloModemControlReadProc;
	ent->write_proc= BuffaloModemControlWriteProc;

	ent = create_proc_entry("buffalo/gpio/mctrl/force_tx", 0, 0);
	ent->read_proc = BuffaloModemControlForceTxReadProc;
	ent->write_proc= BuffaloModemControlForceTxWriteProc;
#endif

#if 0
	if (Buffalo_has_PM()) {
		for (i=0; i<4; i++) {
			sprintf(buf, "buffalo/gpio/power_control/hdd%d", i);
			pde = create_proc_entry(buf, 0, 0);
			if (pde != NULL) {
				pde->read_proc = BuffaloPMHddPowerReadProc;
				pde->write_proc= BuffaloPMHddPowerWriteProc;
				pde->owner = THIS_MODULE;
				pde->data = (void *)i;
			}

			sprintf(buf, "buffalo/gpio/led/pm_diag_led%d", i);
			pde = create_proc_entry(buf, 0, 0);
			if (pde != NULL) {
				pde->read_proc = BuffaloPMHddDiagLedReadProc;
				pde->write_proc= BuffaloPMHddDiagLedWriteProc;
				pde->owner = THIS_MODULE;
				pde->data = (void *)i;
			}
		}
	}
#endif

	printk("%s %s Ver.%s installed.\n", DRIVER_NAME, AUTHOR, BUFFALO_DRIVER_VER);
	return 0;
}

//----------------------------------------------------------------------
void buffaloGpioDriver_exit(void)
{
	int i;
	char buf[1024];
	
	FUNCTRACE(printk(">%s\n",__FUNCTION__));
#ifdef CONFIG_ARCH_FEROCEON_MV78XX0
	remove_proc_entry("buffalo/gpio/mctrl_mode", 0);
#endif

	if (BIT_INFO_LED >= 0) {
		BuffaloLed_BlinkStop(&infoLed);
		remove_proc_entry("buffalo/gpio/led/info", 0);
		remove_proc_entry("buffalo/gpio/led/info_blink", 0);
	}

	if (BIT_PWR_LED >= 0) {
		BuffaloLed_BlinkStop(&powerLed);
		remove_proc_entry("buffalo/gpio/led/power", 0);
		remove_proc_entry("buffalo/gpio/led/power_blink", 0);
	}

	if (BIT_ALARM_LED >= 0) {
		BuffaloLed_BlinkStop(&alarmLed);
		remove_proc_entry("buffalo/gpio/led/alarm", 0);
		remove_proc_entry("buffalo/gpio/led/alarm_blink", 0);
	}

	if (BIT_FAN_LOW >= 0 && BIT_FAN_HIGH >= 0) {
		remove_proc_entry("buffalo/gpio/fan/control", 0);
	}

	if (BIT_FAN_LOCK >= 0) {
		remove_proc_entry("buffalo/gpio/fan/lock", 0);
	}

	remove_proc_entry("buffalo/gpio/fan", 0);

	if (BIT_FUNC_LED >= 0) {
		remove_proc_entry("buffalo/gpio/led/func", 0);
		remove_proc_entry("buffalo/gpio/led/func_blink", 0);
	}

	remove_proc_entry("buffalo/gpio/led/all", 0);
	remove_proc_entry("buffalo/gpio/led", 0);

	if (BIT_FUNC_SW >= 0) {
		remove_proc_entry("buffalo/gpio/switch/func", 0);
	}

	if (BIT_PWRAUTO_SW >= 0) {
		remove_proc_entry("buffalo/gpio/switch/auto_power", 0);
	}

	if (use_slide_power) {
		remove_proc_entry("buffalo/gpio/switch/power", 0);
		remove_proc_entry("buffalo/gpio/switch/sw_control", 0);
	}

	remove_proc_entry("buffalo/gpio/switch", 0);


	for (i=0; mvBoardGpioPinNumGet(BOARD_GPP_HDD_POWER, i) != -1; i++) {
		sprintf(buf, "buffalo/gpio/power_control/hdd%d", i);
		remove_proc_entry(buf, 0);
	}

	for (i=0; mvBoardGpioPinNumGet(BOARD_GPP_USB_VBUS_EN, i) != -1; i++) {
		sprintf(buf, "buffalo/gpio/power_control/usb%d", i);
		remove_proc_entry(buf, 0);
	}

	remove_proc_entry("buffalo/gpio/power_control", 0);
	remove_proc_entry("buffalo/gpio", 0);

	remove_proc_entry ("buffalo", 0);
}

module_init(buffaloGpioDriver_init);
module_exit(buffaloGpioDriver_exit);
